package crontab

import (
	"kpt-pasture/config"
	"kpt-pasture/service/asynqsvc"
	"kpt-pasture/service/redis"
	"kpt-pasture/store/kptstore"

	"gitee.com/xuyiping_admin/pkg/di"
	"go.uber.org/dig"
)

var Module = di.Options(di.Provide(NewCrontab))

type Entry struct {
	dig.In
	Cfg         *config.AppConfig
	DB          *kptstore.DB
	AsynqClient asynqsvc.Client
	Redis       *redis.CacheStoreRedisEntry
}

func NewCrontab(entry Entry) Crontab {
	return &entry
}

type Crontab interface {
	UpdateCowInfo() error
	Indicators() error
	GenerateAsynqWorkOrder() error
	ImmunizationPlan() error
	SameTimePlan() error
	UpdateSameTime() error
	SystemBasicCrontab() error
	DeleteOldOriginal() error
	UpdateDiseaseToCalendar() error

	// NeckRingOriginalMerge 脖环数据
	NeckRingOriginalMerge() error // 合并脖环数据
	NeckRingCalculate() error     // 更新脖环数据
	UpdateCowEstrus() error       // 获取牛只疑似发情数据
	NeckRingEstrusWarning() error // 发情预警
	NeckRingHealthWarning() error // 健康预警

	UpdatePenBehavior() error      // 栏舍行为数据
	UpdatePenBehaviorDaily() error // 栏舍饲养监测
}
